@@ -1,9 +1,8 @@
 use Test::More;
 
-package # Hide from PAUSE
-    Test::DBO;
+package Test::DBO;
 
-use 5.008;
+use 5.010_000;
 use strict;
 use warnings;
 
@@ -24,19 +23,19 @@ BEGIN {
         die $@ if $@;
     }
 
-    # Set up DebugSQL if requested
+    # Set up _Debug_SQL if requested
     require DBIx::DBO::Common;
 
     if ($ENV{DBO_DEBUG_SQL}) {
         diag "DBO_DEBUG_SQL=$ENV{DBO_DEBUG_SQL}";
         package DBIx::DBO::Common;
 
-        DBIx::DBO->config(DebugSQL => $ENV{DBO_DEBUG_SQL});
+        $DBIx::DBO::Common::Config{_Debug_SQL} = $ENV{DBO_DEBUG_SQL};
         no warnings 'redefine';
         *DBIx::DBO::Common::_carp_last_sql = sub {
             my $me = shift;
             my ($cmd, $sql, @bind) = @{$me->_last_sql};
-            local $Carp::Verbose = 1 if $me->config('DebugSQL') > 1;
+            local $Carp::Verbose = 1 if $me->config('_Debug_SQL') > 1;
             my @mess = split /\n/, Carp::shortmess("\t$cmd called");
             splice @mess, 0, 3 if $Carp::Verbose;
             Test::More::diag join "\n", "DEBUG_SQL: $sql", 'DEBUG_SQL: ('.join(', ', map $me->rdbh->quote($_), @bind).')', @mess;
@@ -47,36 +46,17 @@ use DBIx::DBO;
 
 our $dbd;
 our $dbd_name;
-(our $test_db = "DBO_${DBIx::DBO::VERSION}_test_db") =~ s/\W/_/g;
-(our $test_sch = "DBO_${DBIx::DBO::VERSION}_test_sch") =~ s/\W/_/g;
-(our $test_tbl = "DBO_${DBIx::DBO::VERSION}_test_tbl") =~ s/\W/_/g;
+(our $prefix = "DBO_${DBIx::DBO::VERSION}_test") =~ s/\W/_/g;
 our @_cleanup_sql;
 
 sub import {
     my $class = shift;
-    $dbd = shift or return;
+    $dbd = shift;
     $dbd_name = shift;
     my %opt = splice @_;
 
-    grep $_ eq $dbd, DBI->available_drivers
-        or plan skip_all => "No $dbd driver available!";
-
-    # Skip tests with missing module requirements
-    unless (eval { DBIx::DBO->_require_dbd_class($dbd) }) {
-        if ($@ =~ /^Can't load \Q$dbd\E driver\nCan't locate ([\w\/]+)\.pm in \@INC /) {
-            # Module is not installed
-            ($_ = $1) =~ s'/'::'g;
-        } elsif ($@ =~ /^Can't load \Q$dbd\E driver\n([\w:]+ version [\d\.]+) required/) {
-            # Module is not correct version
-            ($_ = $1);
-        } elsif ($@ =~ /^Can't load \Q$dbd\E driver\n\Q$dbd_name\E is not yet supported/) {
-            # DBM is not yet supported
-            plan skip_all => "Can't load $dbd driver: $dbd_name is not yet supported";
-        } else {
-            die $@;
-        }
-        plan skip_all => "Can't load $dbd driver: $_ is required";
-    }
+    grep $_ eq $dbd, DBI->available_drivers or
+        plan skip_all => "No $dbd driver available!";
 
     {
         no strict 'refs';
@@ -94,18 +74,15 @@ sub import {
         }
     }
 
-    if (exists $opt{try_connect}) {
-        try_to_connect($opt{try_connect});
-    }
-
     return unless exists $opt{tests};
 
     if (exists $opt{connect_ok}) {
-        my $dbo = connect_ok(@{$opt{connect_ok}}) or plan skip_all => "Can't connect: $DBI::errstr";
+        my $dbo_ref = shift @{$opt{connect_ok}};
+        $$dbo_ref = connect_dbo(@{$opt{connect_ok}}) or plan skip_all => "Can't connect: $DBI::errstr";
 
         plan tests => $opt{tests};
         pass "Connect to $dbd_name";
-        isa_ok $dbo, "DBIx::DBO::DBD::${dbd}", '$dbo';
+        isa_ok $$dbo_ref, "DBIx::DBO::DBD::${dbd}::Handle", '$dbo';
     } else {
         plan tests => $opt{tests};
     }
@@ -124,36 +101,25 @@ sub sql_err {
     join "\n", @err;
 }
 
-sub last_sql {
-    my $me = shift;
-    my ($cmd, $sql, @bind) = @{$me->_last_sql};
-    print $sql, "\n(".join(', ', map $me->rdbh->quote($_), @bind).")\n";
+sub connect_ok {
+    ok my $dbo = connect_dbo(@_), "Connect to $dbd_name" or die $DBI::errstr;
+    isa_ok $dbo, "DBIx::DBO::DBD::${dbd}::Handle", '$dbo';
+    return $dbo;
 }
 
 sub connect_dbo {
-    my ($dsn, $user, $pass) = @_;
-    defined $dsn or $dsn = '';
+    my $dsn = shift // $ENV{'DBO_TEST_'.uc($dbd).'_DB'} // '';
+    my $user = shift // $ENV{'DBO_TEST_'.uc($dbd).'_USER'};
+    my $pass = shift // $ENV{'DBO_TEST_'.uc($dbd).'_PASS'};
+
     DBIx::DBO->connect("DBI:$dbd:$dsn", $user, $pass, {RaiseError => 0});
 }
 
-sub try_to_connect {
-    my $dbo_ref = shift;
-    my @env = map $ENV{"DBO_TEST_\U$dbd\E_$_"}, qw(DSN USER PASS);
-    if (grep defined, @env) {
-        return $$dbo_ref if $$dbo_ref = connect_dbo(@env);
-        plan skip_all => "Can't connect: $DBI::errstr";
-    }
-    return undef;
-}
-
-sub connect_ok {
-    my $dbo_ref = shift;
-    return try_to_connect($dbo_ref) || ($$dbo_ref = connect_dbo(@_));
-}
-
 sub basic_methods {
     my $dbo = shift;
-    my $quoted_table = $dbo->_qi($test_sch, $test_tbl);
+    my $schema = shift;
+    my $table = shift;
+    my $quoted_table = $dbo->_qi($schema, $table);
     my $t;
 
     # Create a test table with a multi-column primary key
@@ -161,7 +127,7 @@ sub basic_methods {
         pass 'Create a test table';
 
         # Create a table object
-        $t = $dbo->table([$test_sch, $test_tbl]);
+        $t = $dbo->table([$schema, $table]);
         isa_ok $t, 'DBIx::DBO::Table', '$t';
 
         # Check the Primary Keys
@@ -171,7 +137,7 @@ sub basic_methods {
         $dbo->do("DROP TABLE $quoted_table") && $dbo->do("CREATE TABLE $quoted_table (id INT, name TEXT)")
             or diag sql_err($dbo) or die "Can't recreate the test table!\n";
         $dbo->_get_table_info($t->{Schema}, $t->{Name});
-        $t = $dbo->table([$test_sch, $test_tbl]);
+        $t = $dbo->table([$schema, $table]);
     }
     else {
         diag sql_err($dbo);
@@ -184,11 +150,9 @@ sub basic_methods {
             or diag sql_err($dbo) or die "Can't create the test table!\n";
 
         # Create our table object
-        $t = $dbo->table([$test_sch, $test_tbl]);
+        $t = $dbo->table([$schema, $table]);
         isa_ok $t, 'DBIx::DBO::Table', '$t';
     }
-    die "Couldn't create the DBIx::DBO::Table object!" unless $t;
-
     pass 'Method DBIx::DBO->do';
 
     # Insert data
@@ -236,7 +200,7 @@ sub basic_methods {
     is $rv, 1, 'Method DBIx::DBO::Table->delete';
 
     # Remove the created table during cleanup
-    todo_cleanup("DROP TABLE $quoted_table");
+    push @_cleanup_sql, "DROP TABLE $quoted_table";
 
     return $t;
 }
@@ -326,10 +290,7 @@ sub query_methods {
     # Fetch the first row
     $r = $q->fetch;
     ok $r->isa('DBIx::DBO::Row'), 'Method DBIx::DBO::Query->fetch';
-    SKIP: {
-        skip "Re-use row doesn't work with Devel::Cover", 1 if exists $INC{'Devel/Cover.pm'};
-        is $r_str, "$r", 'Re-use the same row object';
-    }
+    is $r_str, "$r", 'Re-use the same row object';
 
     # Access methods
     is $r->{name}, 'John Doe', 'Access row as a hashref';
@@ -348,22 +309,6 @@ sub query_methods {
     1 while $q->fetch;
     is $q->rows, 6, 'Row count is 6';
 
-    # Parentheses
-    $q->where('name', 'LIKE', \"'%o%'");
-    $q->open_bracket('OR');
-    $q->where('name', 'LIKE', \"'%a%'");
-    $q->where('id', '!=', \1);
-    $q->close_bracket;
-    is_deeply $q->col_arrayref({ Columns => [1] }), [4,5,6], 'Method DBIx::DBO::Query->open_bracket';
-
-    # Reset the Query
-    $q->reset;
-    is $q->sql, $dbo->query($t)->sql, 'Method DBIx::DBO::Query->reset';
-
-    # Group by the first initial
-    $q->show({FUNC => 'SUBSTR(?, 1, 1)', COL => 'name', AS => 'initial'});
-    ok(($q->group_by('initial'), $q->run), 'Method DBIx::DBO::Query->group_by') or diag sql_err($q);
-
     $q->finish;
     return $q;
 }
@@ -372,11 +317,9 @@ sub advanced_query_methods {
     my $dbo = shift;
     my $t = shift;
     my $q = shift;
-    $q->reset;
 
     # Show specific columns only
     $q->show({ FUNC => 'UPPER(?)', COL => 'name', AS => 'name' }, 'id', 'name');
-    $q->order_by('id');
     is $q->fetch->{name}, 'JOHN DOE', 'Method DBIx::DBO::Query->show';
     is $q->row ** $t ** 'name', 'John Doe', 'Access specific column';
 
@@ -389,7 +332,7 @@ sub advanced_query_methods {
     $case_sensitive = $case_sensitive->[0];
     note "$dbd_name 'LIKE' is".($case_sensitive ? '' : ' NOT').' case sensitive';
 
-    # WHERE clause
+    # Where clause
     $q->show('id');
     ok $q->where('name', 'LIKE', '%a%'), 'Method DBIx::DBO::Query->where LIKE';
     my $a = $q->col_arrayref or diag sql_err($q);
@@ -401,10 +344,6 @@ sub advanced_query_methods {
     $a = $q->hashref('id') or diag sql_err($q);
     is_deeply $a, {4 => {id => 4},6 => {id => 6}}, 'Method DBIx::DBO::Query->hashref';
 
-    # HAVING clause
-    $q->show('id', 'name', { FUNC => 'CONCAT(?, ?)', COL => [qw(id name)], AS => 'combo'});
-    ok $q->having('combo', '=', '4James Bond'), 'Method DBIx::DBO::Query->having';
-
     $q->finish;
 }
 
@@ -460,11 +399,6 @@ sub join_methods {
     $q->finish;
 }
 
-sub todo_cleanup {
-    my $sql = shift;
-    unshift @_cleanup_sql, $sql;
-}
-
 sub cleanup {
     my $dbo = shift;
 
@@ -472,9 +406,6 @@ sub cleanup {
     for my $sql (@_cleanup_sql) {
         $dbo->do($sql) or diag sql_err($dbo);
     }
-
-    $dbo->disconnect;
-    ok !defined $dbo->{dbh} && !defined $dbo->{rdbh}, 'Method DBIx::DBO->disconnect';
 }
 
 my @_no_recursion;
@@ -492,18 +423,18 @@ sub Dump {
             $var = 'r';
         }
     }
-    defined $var or $var = 'dump';
+    $var //= 'dump';
     require Data::Dumper;
     my $d = Data::Dumper->new([$val], [$var]);
     my %seen;
-    if (defined $val) {
-        @_no_recursion = ($val);
-        if (reftype $val eq 'ARRAY')   { _Find_Seen(\%seen, $_) for @$val }
-        elsif (reftype $val eq 'HASH') { _Find_Seen(\%seen, $_) for values %$val }
-        elsif (reftype $val eq 'REF')  { _Find_Seen(\%seen, $$val) }
-        $d->Seen(\%seen);
+    @_no_recursion = ($val);
+    given (reftype $val) {
+        when ('ARRAY') { _Find_Seen(\%seen, $_) for @$val }
+        when ('HASH')  { _Find_Seen(\%seen, $_) for values %$val }
+        when ('REF')   { _Find_Seen(\%seen, $$val) }
     }
-    print $d->Dump;
+    $d->Seen(\%seen);
+    warn $d->Dump;
 }
 
 sub _Find_Seen {
@@ -536,9 +467,11 @@ sub _Find_Seen {
             return;
         }
     }
-    if (reftype $val eq 'ARRAY')   { _Find_Seen($seen, $_) for @$val }
-    elsif (reftype $val eq 'HASH') { _Find_Seen($seen, $_) for values %$val }
-    elsif (reftype $val eq 'REF')  { _Find_Seen($seen, $$val) }
+    given (reftype $val) {
+        when ('ARRAY') { _Find_Seen($seen, $_) for @$val }
+        when ('HASH')  { _Find_Seen($seen, $_) for values %$val }
+        when ('REF')   { _Find_Seen($seen, $$val) }
+    }
 }
 
 1;
