@@ -1,15 +1,37 @@
-package # hide from PAUSE
-    DBIx::DBO::Common;
+package DBIx::DBO::Common;
 
-use 5.008;
+use 5.010;
 use strict;
 use warnings;
 use Carp;
 use Scalar::Util 'blessed';
 use constant PLACEHOLDER => "\x{b1}\x{a4}\x{221e}";
 
-# Common routines and variables exported to all DBO classes.
-# This module automatically exports ALL the methods and variables for use in the other DBO modules.
+=head1 NAME
+
+DBIx::DBO::Common - Common routines and variables exported to all DBO classes.
+
+=head1 DESCRIPTION
+
+This module automatically exports ALL the methods and variables for use in the other DBO modules.
+
+=head2 do
+
+  $dbo->do($statement)         or die $dbo->dbh->errstr;
+  $dbo->do($statement, \%attr) or die $dbo->dbh->errstr;
+  $dbo->do($statement, \%attr, @bind_values) or die ...
+
+This provides access to DBI C<do> method.
+
+=head2 dbh
+
+The read-write DBI handle.
+
+=head2 rdbh
+
+The read-only DBI handle, or if there is no read-only connection, the read-write DBI handle.
+
+=cut
 
 use subs qw(ouch oops);
 *oops = \&Carp::carp;
@@ -17,7 +39,7 @@ use subs qw(ouch oops);
 
 our %Config = (
     QuoteIdentifier => 1,
-    DebugSQL => 0,
+    _Debug_SQL => 0,
 );
 our @CARP_NOT;
 our $placeholder = PLACEHOLDER;
@@ -41,7 +63,7 @@ sub _qi {
     my $me = shift;
     return $me->dbh->quote_identifier(@_) if $me->config('QuoteIdentifier');
     # Strip off any null/undef elements (ie schema)
-    shift while @_ and not (defined $_[0] and length $_[0]);
+    shift while @_ and !defined $_[0];
     join '.', @_;
 }
 
@@ -55,7 +77,7 @@ sub _last_sql {
 sub _carp_last_sql {
     my $me = shift;
     my ($cmd, $sql, @bind) = @{$me->_last_sql};
-    local $Carp::Verbose = 1 if $me->config('DebugSQL') > 1;
+    local $Carp::Verbose = 1 if $me->config('_Debug_SQL') > 1;
     my @mess = split /\n/, Carp::shortmess("\t$cmd called");
     splice @mess, 0, 3 if $Carp::Verbose;
     warn join "\n", $sql, '('.join(', ', map $me->rdbh->quote($_), @bind).')', @mess;
@@ -65,7 +87,7 @@ sub _sql {
     my ($me, $sql) = splice @_, 0, 2;
     my $cmd = (caller(1))[3];
     $me->_last_sql($cmd, $sql, @_);
-    $me->_carp_last_sql if $me->config('DebugSQL');
+    $me->_carp_last_sql if $me->config('_Debug_SQL');
 }
 
 sub do {
@@ -81,7 +103,6 @@ sub _build_sql_select {
     $sql .= ' FROM '.$me->_build_from($h);
     $sql .= ' WHERE '.$_ if $_ = $me->_build_where($h);
     $sql .= ' GROUP BY '.$_ if $_ = $me->_build_group($h);
-    $sql .= ' HAVING '.$_ if $_ = $me->_build_having($h);
     $sql .= ' ORDER BY '.$_ if $_ = $me->_build_order($h);
     $sql .= ' '.$_ if $_ = $me->_build_limit($h);
     $sql;
@@ -92,7 +113,7 @@ sub _bind_params_select {
     my $h = shift;
     map {
         exists $h->{$_} ? @{$h->{$_}} : ()
-    } qw(Show_Bind From_Bind Where_Bind Group_Bind Having_Bind Order_Bind);
+    } qw(Show_Bind From_Bind Where_Bind Group_Bind Order_Bind);
 }
 
 # TODO: Should we die if GROUP BY is set?
@@ -161,31 +182,30 @@ sub _build_from {
     my $h = shift;
     return $h->{from} if defined $h->{from};
     undef @{$h->{From_Bind}};
-    $h->{from} = $me->_build_table(($me->tables)[0]);
-    for (my $i = 1; $i < $me->tables; $i++) {
-        $h->{from} .= $h->{Join}[$i].$me->_build_table(($me->tables)[$i]);
-        if ($h->{Join_On}[$i]) {
-            $h->{from} .= ' ON '.join(' AND ', $me->_build_where_chunk($h->{From_Bind}, 'OR', $h->{Join_On}[$i]));
+    $h->{from} = $me->_build_table(($me->_tables)[0]);
+    for (my $i = 1; $i < $me->_tables; $i++) {
+        $h->{from} .= $h->{Join}[$i].$me->_build_table(($me->_tables)[$i]);
+        if ($h->{JoinOn}[$i]) {
+            $h->{from} .= ' ON '.join(' AND ', $me->_build_where_chunk($h->{From_Bind}, 'OR', $h->{JoinOn}[$i]));
         }
     }
     $h->{from};
 }
 
-sub _valid_col {
-    my ($me, $col) = @_;
-    for my $tbl ($me->tables) {
-        return $col if $col->[0] == $tbl;
-    }
-    ouch 'Invalid column, the column is from a table not included in this query';
-}
-
 sub _parse_col {
-    my ($me, $col, $_check_aliases) = @_;
-    if (ref $col) {
-        return $me->_valid_col($col) if blessed $col and $col->isa('DBIx::DBO::Column');
-        ouch 'Invalid column: '.$col;
+    my ($me, $col) = @_;
+    if (blessed $col and $col->isa('DBIx::DBO::Column')) {
+        for my $tbl ($me->_tables) {
+            return $col if $col->[0] == $tbl;
+        }
+        # TODO: Flesh out this ouch a bit
+        ouch 'Invalid table';
     }
-    $me->column($col, $_check_aliases);
+    ouch 'Invalid column: '.$col if ref $col;
+    for my $tbl ($me->_tables) {
+        return $tbl->column($col) if exists $tbl->{Column_Idx}{$col};
+    }
+    ouch 'No such column: '.$col;
 }
 
 sub _build_col {
@@ -196,12 +216,12 @@ sub _build_col {
 sub _parse_val {
     my $me = shift;
     my $fld = shift;
-    my %c = (Check => '', @_);
+    my $check_fld = shift || '';
 
     my $func;
     my %opt;
     if (ref $fld eq 'SCALAR') {
-        ouch 'Invalid '.($c{Check} eq 'Column' ? 'column' : 'field').' reference (scalar ref to undef)'
+        ouch 'Invalid '.($check_fld eq 'Column' ? 'column' : 'field').' reference (scalar ref to undef)'
             unless defined $$fld;
         $func = $$fld;
         $fld = [];
@@ -215,13 +235,10 @@ sub _parse_val {
         $opt{COLLATE} = $fld->{COLLATE} if exists $fld->{COLLATE};
         if (exists $fld->{COL}) {
             ouch 'Invalid HASH containing both COL and VAL' if exists $fld->{VAL};
-            my @cols = ref $fld->{COL} eq 'ARRAY' ? @{$fld->{COL}} : $fld->{COL};
-            $fld = [ map $me->_parse_col($_, $c{Aliases}), @cols ];
+            $fld = $me->_parse_col($fld->{COL});
         } else {
             $fld = exists $fld->{VAL} ? $fld->{VAL} : [];
         }
-    } elsif (blessed $fld and $fld->isa('DBIx::DBO::Column')) {
-        return [ $me->_valid_col($fld) ];
     }
     $fld = [$fld] unless ref $fld eq 'ARRAY';
 
@@ -230,8 +247,8 @@ sub _parse_val {
     if (defined $func) {
         my $need = $me->_substitute_placeholders($func);
         ouch "The number of params ($with) does not match the number of placeholders ($need)" if $need != $with;
-    } elsif ($with != 1 and $c{Check} ne 'Auto') {
-        ouch 'Invalid '.($c{Check} eq 'Column' ? 'column' : 'field')." reference (passed $with params instead of 1)";
+    } elsif ($with != 1 and $check_fld ne 'Auto') {
+        ouch 'Invalid '.($check_fld eq 'Column' ? 'column' : 'field')." reference (passed $with params instead of 1)";
     }
     return ($fld, $func, \%opt);
 }
@@ -274,7 +291,6 @@ sub _build_val {
     return $func.$extra;
 }
 
-# Construct the WHERE clause
 sub _build_where {
     my $me = shift;
     my $h = shift;
@@ -286,7 +302,6 @@ sub _build_where {
     $h->{where} = join ' AND ', @where;
 }
 
-# Construct the WHERE contents of one set of parentheses
 sub _build_where_chunk {
     my $me = shift;
     my ($bind, $ag, $whs) = @_;
@@ -306,13 +321,13 @@ sub _build_where_chunk {
                     # Right now this starts with the last @whs and works backwards
                     # It splices when the ag is the correct AND/OR and the funcs match and all flds match
                     next if (ref $whs[$i][0] or $ag ne ($whs[$i][7] || _op_ag($whs[$i][0])));
-                    no warnings 'uninitialized';
-                    next if $whs[$i][2] ne $fld_func;
-                    use warnings 'uninitialized';
-#                    next unless $fld_func ~~ $whs[$i][2];
-                    my $l = $whs[$i][1];
-                    next if ((ref $l eq 'ARRAY' ? "@$l" : $l) ne (ref $fld eq 'ARRAY' ? "@$fld" : $fld));
-#                    next unless $fld ~~ $whs[$i][1];
+#                    no warnings 'uninitialized';
+#                    next if $whs[$i][2] ne $fld_func;
+#                    use warnings 'uninitialized';
+                    next unless $fld_func ~~ $whs[$i][2];
+#                    my $l = $whs[$i][1];
+#                    next if ((ref $l eq 'ARRAY' ? "@$l" : $l) ne (ref $fld eq 'ARRAY' ? "@$fld" : $fld));
+                    next unless $fld ~~ $whs[$i][1];
                     push @ary, $me->_build_where_piece($bind, @{splice @whs, $i, 1});
                 }
             }
@@ -327,13 +342,11 @@ sub _op_ag {
     return 'AND' if $_[0] eq '!=' or $_[0] eq 'IS NOT' or $_[0] eq '<>' or $_[0] eq 'NOT IN' or $_[0] eq 'NOT BETWEEN';
 }
 
-# Construct one WHERE expression
 sub _build_where_piece {
     my ($me, $bind, $op, $fld, $fld_func, $fld_opt, $val, $val_func, $val_opt) = @_;
     $me->_build_val($bind, $fld, $fld_func, $fld_opt)." $op ".$me->_build_val($bind, $val, $val_func, $val_opt);
 }
 
-# Construct one WHERE expression (simple)
 sub _build_quick_where {
     ouch 'Wrong number of arguments' if @_ & 1;
     my ($me, $bind) = splice @_, 0, 2;
@@ -368,17 +381,6 @@ sub _build_group {
     $h->{group} = join ', ', map $me->_build_val($h->{Group_Bind}, @$_), @{$h->{GroupBy}};
 }
 
-# Construct the HAVING clause
-sub _build_having {
-    my $me = shift;
-    my $h = shift;
-    return $h->{having} if defined $h->{having};
-    undef @{$h->{Having_Bind}};
-    my @having;
-    push @having, $me->_build_where_chunk($h->{Having_Bind}, 'OR', $h->{Having_Data}) if exists $h->{Having_Data};
-    $h->{having} = join ' AND ', @having;
-}
-
 sub _build_order {
     my $me = shift;
     my $h = shift;
@@ -397,30 +399,4 @@ sub _build_limit {
     $h->{limit};
 }
 
-sub _set_config {
-    my $me = shift;
-    my ($ref, $opt, $val) = @_;
-    ouch "Invalid value for the 'UseHandle' setting"
-        if $opt eq 'UseHandle' and $val and $val ne 'read-only' and $val ne 'read-write';
-    my $old = $ref->{$opt};
-    $ref->{$opt} = $val;
-    return $old;
-}
-
-sub _set_dbd_inheritance {
-    my $class = shift;
-    my $dbd = shift;
-    $class =~ s/::DBD::\w+$//;
-    # Inheritance
-    no strict 'refs';
-    unless (@{$class.'::DBD::'.$dbd.'::ISA'}) {
-        my @isa = grep $_->can('_set_dbd_inheritance'), @_ ? @_ : @{$class.'::ISA'};
-        $_->_set_dbd_inheritance($dbd) for @isa;
-        @{$class.'::DBD::'.$dbd.'::ISA'} = ($class, map $_.'::DBD::'.$dbd, @isa);
-        mro::set_mro($class.'::DBD::'.$dbd, 'c3');
-        Class::C3::initialize() if $] < 5.009_005;
-    }
-    return $class.'::DBD::'.$dbd;
-}
-
 1;
