@@ -1,12 +1,10 @@
 use strict;
 use warnings;
-
-BEGIN { die "DBM is not yet supported!\n" unless $ENV{DBO_ALLOW_DBM} }
 use SQL::Statement;
 
+die "DBM is not yet supported!\n" unless $ENV{DBO_ALLOW_DBM};
 
-package # hide from PAUSE
-    DBIx::DBO::DBD::DBM;
+package DBIx::DBO::DBD::DBM::Handle;
 use DBIx::DBO::Common;
 
 sub _bless_dbo {
@@ -20,8 +18,9 @@ sub _bless_dbo {
 sub _get_table_schema {
     my $me = shift;
     my $schema = shift; # Not used
-    my $table = shift;
+    my $table = my $q_table = shift;
     ouch 'No table name supplied' unless defined $table and length $table;
+
     return;
 }
 
@@ -49,7 +48,7 @@ sub _get_table_info {
     }
     $h{Columns} = [ sort { $h{Column_Idx}{$a} cmp $h{Column_Idx}{$b} } keys %{$h{Column_Idx}} ];
     $h{PrimaryKeys} = [];
-    $me->{TableInfo}{defined $schema ? $schema : ''}{$table} = \%h;
+    $me->{TableInfo}{$schema // ''}{$table} = \%h;
 }
 
 1;
