@@ -1,8 +1,7 @@
 use strict;
 use warnings;
 
-package # hide from PAUSE
-    DBIx::DBO::DBD::Pg;
+package DBIx::DBO::DBD::Pg::Handle;
 use DBIx::DBO::Common;
 
 sub _get_table_schema {
@@ -45,7 +44,7 @@ sub _get_table_info {
     } else {
         $h{PrimaryKeys} = [];
     }
-    $me->{TableInfo}{defined $schema ? $schema : ''}{$table} = \%h;
+    $me->{TableInfo}{$schema // ''}{$table} = \%h;
 }
 
 sub table_info {
