@@ -2,8 +2,7 @@ use strict;
 use warnings;
 use DBD::SQLite 1.27;
 
-package # hide from PAUSE
-    DBIx::DBO::DBD::SQLite;
+package DBIx::DBO::DBD::SQLite::Handle;
 use DBIx::DBO::Common;
 
 sub _get_table_schema {
@@ -22,8 +21,7 @@ sub _get_table_schema {
     return $info->[0][1];
 }
 
-package # hide from PAUSE
-    DBIx::DBO::DBD::SQLite::Query;
+package DBIx::DBO::DBD::SQLite::Query;
 use DBIx::DBO::Common;
 
 sub fetch {
