@@ -8,64 +8,34 @@ use warnings;
 use overload '@{}' => sub {${$_[0]}->{array} || []}, '%{}' => sub {${$_[0]}->{hash}};
 use overload '**' => \&value, fallback => 1;
 
-our @ISA;
-
-=head1 NAME
-
-DBIx::DBO::Row - An OO interface to SQL queries and results.  Encapsulates a fetched row of data in an object.
-
-=head1 SYNOPSIS
-
-  # Create a Row object for the `users` table
-  my $row = $dbo->row('users');
-  
-  # Load my record
-  $row->load(login => 'vlyon') or die "Where am I?";
-  
-  # Double my salary :)
-  $row->update(salary => {FUNC => '? * 2', COL => 'salary'});
-  
-  # Print my email address
-  print $row ** 'email';  # Short for: $row->value('email')
-  
-  # Delete my boss
-  $row->load(id => $row ** boss_id)->delete or die "Can't kill the boss";
-
-=head1 METHODS
-
-=head3 C<new>
-
-  DBIx::DBO::Row->new($dbo, $table_object);
-  DBIx::DBO::Row->new($dbo, $query_object);
-
-Create and return a new C<Row> object.
-
-=cut
-
 sub dbh { ${$_[0]}->{DBO}->dbh }
 sub rdbh { ${$_[0]}->{DBO}->rdbh }
 
-sub new {
+=head2 config
+
+  $parent_setting = $dbo->config($option)
+  $dbo->config($option => $parent_setting)
+
+Get or set the parent (if it has one) or dbo or global config settings.
+When setting an option, the previous value is returned.
+
+=cut
+
+sub config {
+    my $me = shift;
+    ($$me->{Parent} // $$me->{DBO})->config(@_);
+}
+
+sub _new {
     my $proto = shift;
     my $class = ref($proto) || $proto;
     my $me = \{ DBO => shift, Parent => shift, array => undef, hash => {} };
-    blessed $$me->{DBO} and $$me->{DBO}->isa('DBIx::DBO') or ouch 'Invalid DBO Object';
+    blessed $$me->{DBO} and $$me->{DBO}->isa('DBIx::DBO::Handle') or ouch 'Invalid DBO Object';
     ouch 'Invalid Parent Object' unless defined $$me->{Parent};
     $$me->{Parent} = $$me->{DBO}->table($$me->{Parent}) unless blessed $$me->{Parent};
-    bless $me, $class->_create_dbd_class($$me->{DBO}{dbd});
-    Class::C3::initialize() if $DBIx::DBO::need_c3_initialize;
-    $me->_init;
-    return wantarray ? ($me, $me->tables) : $me;
-}
-
-*_create_dbd_class = \&DBIx::DBO::Common::_create_dbd_class;
-
-sub _set_dbd_inheritance {
-    my $class = shift;
-    my $dbd = shift;
-    # Let DBIx::DBO::Row secretly inherit from DBIx::DBO::Common
-    @_ = (@ISA, 'DBIx::DBO::Common') if not @_ and $class eq __PACKAGE__;
-    $class->DBIx::DBO::Common::_set_dbd_inheritance($dbd, @_);
+    _init($me);
+    bless $me, $class;
+    return wantarray ? ($me, $me->_tables) : $me;
 }
 
 sub _init {
@@ -73,7 +43,7 @@ sub _init {
     $$me->{build_data}{LimitOffset} = [1];
     if ($$me->{Parent}->isa('DBIx::DBO::Query')) {
         $$me->{Tables} = [ @{$$me->{Parent}{Tables}} ];
-        $me->_copy_build_data;
+        _copy_build_data($me);
         # We must weaken this to avoid a circular reference
         weaken $$me->{Parent};
     } elsif ($$me->{Parent}->isa('DBIx::DBO::Table')) {
@@ -98,13 +68,7 @@ sub _copy_build_data {
     }
 }
 
-=head3 C<tables>
-
-Return a list of L<DBIx::DBO::Table|DBIx::DBO::Table> objects for this row.
-
-=cut
-
-sub tables {
+sub _tables {
     my $me = shift;
     @{$$me->{Tables}};
 }
@@ -124,11 +88,16 @@ sub _table_alias {
     @{$$me->{Tables}} > 1 ? 't'.($i + 1) : ();
 }
 
+sub _showing {
+    my $me = shift;
+    @{$$me->{build_data}{Showing}} ? @{$$me->{build_data}{Showing}} : @{$$me->{Tables}};
+}
+
 sub _column_idx {
     my $me = shift;
     my $col = shift;
     my $idx = -1;
-    for my $shown (@{$$me->{build_data}{Showing}} ? @{$$me->{build_data}{Showing}} : @{$$me->{Tables}}) {
+    for my $shown ($me->_showing) {
         if (blessed $shown and $shown->isa('DBIx::DBO::Table')) {
             if ($col->[0] == $shown and exists $shown->{Column_Idx}{$col->[1]}) {
                 return $idx + $shown->{Column_Idx}{$col->[1]};
@@ -142,41 +111,12 @@ sub _column_idx {
     return;
 }
 
-=head3 C<column>
+=head2 value
 
-  $query->column($column_name);
-  $query->column($column_or_alias_name, 1);
+  $row->value($column)
 
-Returns a reference to a column for use with other methods.
-
-=cut
-
-sub column {
-    my ($me, $col, $_check_aliases) = @_;
-    if ($_check_aliases) {
-        for my $fld (@{$$me->{build_data}{Showing}}) {
-            return $$me->{Column}{$col} ||= bless [$me, $col], 'DBIx::DBO::Column'
-                if !blessed $fld and exists $fld->[2]{AS} and $col eq $fld->[2]{AS};
-        }
-    }
-    for my $tbl ($me->tables) {
-        return $tbl->column($col) if exists $tbl->{Column_Idx}{$col};
-    }
-    ouch 'No such column'.($_check_aliases ? '/alias' : '').': '.$me->_qi($col);
-}
-
-=head3 C<value>
-
-  $value = $row->value($column);
-  $value = $row ** $column;
-
-Return the value in the C<$column> field.  The C<**> method is a shortcut for the C<value> method.
-C<$column> can be a column name or a C<Column> object.
-
-Values in the C<Row> can also be obtained by using the object as an array/hash reference.
-
-  $value = $row->[2];
-  $value = $row->{some_column};
+Return the value in this field.
+$column can be a column name or a column object.
 
 =cut
 
@@ -193,13 +133,13 @@ sub value {
     ouch 'No such column: '.$col;
 }
 
-=head3 C<load>
+=head2 load
 
   $row->load(id => 123);
   $row->load(name => 'Bob', status => 'Employed');
 
 Fetch a new row using the where definition specified.
-Returns the C<Row> object if the row is found and loaded successfully.
+Returns the Row object if the row is found and loaded successfully.
 Returns an empty list if there is no row or an error occurs.
 
 =cut
@@ -209,7 +149,7 @@ sub load {
 
     $me->_detach;
 
-    # Use Quick_Where to load a row, but make sure to restore its value afterward
+    # TODO: Shouldn't this replace the Quick_Where?
     my $old_qw = $#{$$me->{build_data}{Quick_Where}};
     push @{$$me->{build_data}{Quick_Where}}, @_;
     undef $$me->{build_data}{where};
@@ -237,24 +177,21 @@ sub _detach {
         $$me->{array} = [ @$me ];
         $$me->{hash} = { %$me };
         undef $$me->{Parent}{Row};
-        # Save config from Parent
-        if ($$me->{Parent}{Config} and %{$$me->{Parent}{Config}}) {
-            $$me->{Config} = { %{$$me->{Parent}{Config}}, $$me->{Config} ? %{$$me->{Config}} : () };
-        }
+        # TODO: Save configs from Parent
     }
     undef $$me->{Parent};
 }
 
-=head3 C<update>
+=head2 update
 
   $row->update(id => 123);
   $row->update(name => 'Bob', status => 'Employed');
 
 Updates the current row with the new values specified.
-Returns the number of rows updated or C<'0E0'> for no rows to ensure the value is true,
+Returns the number of rows updated or '0E0' for no rows to ensure the value is true,
 and returns false if there was an error.
 
-Note: If C<LIMIT> is supported on C<UPDATE>s then only the first matching row will be updated
+Note: If LIMIT is supported on UPDATEs then only the first matching row will be updated
 otherwise ALL rows matching the current row will be updated.
 
 =cut
@@ -263,31 +200,25 @@ sub update {
     my $me = shift;
     ouch 'No current record to update!' unless $$me->{array};
     my $build_data = $me->_build_data_matching_this_row;
-    $build_data->{LimitOffset} = [1] if $me->config('LimitRowUpdate') and $me->tables == 1;
+    $build_data->{LimitOffset} = [1] if $me->config('LimitRowUpdate') and $me->_tables == 1;
     my $sql = $me->_build_sql_update($build_data, @_);
 
-    my $rv = $me->do($sql, undef, $me->_bind_params_update($build_data));
-    $me->_reset_on_update($build_data, @_) if $rv and $rv > 0;
-    return $rv;
-}
-
-sub _reset_on_update {
-    my $me = shift;
     # TODO: Reload/update instead of leaving the row empty?
     # To update the Row object is difficult because columns may have been aliased
     undef $$me->{array};
     undef %$me;
+    $me->do($sql, undef, $me->_bind_params_update($build_data));
 }
 
-=head3 C<delete>
+=head2 delete
 
   $row->delete;
 
 Deletes the current row.
-Returns the number of rows deleted or C<'0E0'> for no rows to ensure the value is true,
+Returns the number of rows deleted or '0E0' for no rows to ensure the value is true,
 and returns false if there was an error.
 
-Note: If C<LIMIT> is supported on C<DELETE>s then only the first matching row will be deleted
+Note: If LIMIT is supported on DELETEs then only the first matching row will be deleted
 otherwise ALL rows matching the current row will be deleted.
 
 =cut
@@ -296,7 +227,7 @@ sub delete {
     my $me = shift;
     ouch 'No current record to delete!' unless $$me->{array};
     my $build_data = $me->_build_data_matching_this_row;
-    $build_data->{LimitOffset} = [1] if $me->config('LimitRowDelete') and $me->tables == 1;
+    $build_data->{LimitOffset} = [1] if $me->config('LimitRowDelete') and $me->_tables == 1;
     my $sql = $me->_build_sql_delete($build_data, @_);
 
     undef $$me->{array};
@@ -307,73 +238,20 @@ sub delete {
 sub _build_data_matching_this_row {
     my $me = shift;
     # Identify the row by the PrimaryKeys if any, otherwise by all Columns
-    my @quick_where;
+    my @cols;
     for my $tbl (@{$$me->{Tables}}) {
-        for my $col (map $tbl ** $_, @{$tbl->{ @{$tbl->{PrimaryKeys}} ? 'PrimaryKeys' : 'Columns' }}) {
-            my $i = $me->_column_idx($col);
-            defined $i or ouch 'The '.$me->_qi($tbl->{Name}, $col->[1]).' field needed to identify this row, was not included in this query';
-            push @quick_where, $col => $$me->{array}[$i];
-        }
+        push @cols, map $tbl ** $_, @{$tbl->{ @{$tbl->{PrimaryKeys}} ? 'PrimaryKeys' : 'Columns' }};
     }
     my %h = (
         from => $$me->{build_data}{from},
-        Quick_Where => \@quick_where
+        Quick_Where => [ map {$_ => $me->value($_)} @cols ]
     );
     $h{From_Bind} = $$me->{build_data}{From_Bind} if exists $$me->{build_data}{From_Bind};
     return \%h;
 }
 
-=head2 Common Methods
-
-These methods are accessible from all DBIx::DBO* objects.
-
-=head3 C<dbh>
-
-The I<read-write> C<DBI> handle.
-
-=head3 C<rdbh>
-
-The I<read-only> C<DBI> handle, or if there is no I<read-only> connection, the I<read-write> C<DBI> handle.
-
-=head3 C<do>
-
-  $dbo->do($statement)         or die $dbo->dbh->errstr;
-  $dbo->do($statement, \%attr) or die $dbo->dbh->errstr;
-  $dbo->do($statement, \%attr, @bind_values) or die ...
-
-This provides access to the L<DBI-E<gt>do|DBI/"do"> method.  It defaults to using the I<read-write> C<DBI> handle.
-
-=head3 C<config>
-
-  $row_setting = $dbo->config($option);
-  $dbo->config($option => $row_setting);
-
-Get or set the C<Row> config settings.  When setting an option, the previous value is returned.  When getting an option's value, if the value is undefined, the C<Query> object (If the the C<Row> belongs to one) or L<DBIx::DBO|DBIx::DBO>'s value is returned.
-
-See L<DBIx::DBO/Available_config_options>.
-
-=cut
-
-sub config {
-    my $me = shift;
-    my $opt = shift;
-    return $me->_set_config($$me->{Config} ||= {}, $opt, shift) if @_;
-    return defined $$me->{Config}{$opt} ? $$me->{Config}{$opt} :
-        (defined $$me->{Parent} ? $$me->{Parent} : $$me->{DBO})->config($opt);
-}
-
 sub DESTROY {
     undef %${$_[0]};
 }
 
 1;
-
-__END__
-
-=head1 SEE ALSO
-
-L<DBIx::DBO>
-
-
-=cut
-
